==> mandrel-23.0.0.0-Final.tar.gz: mandrel-mandrel-23.0.0.0-Final/espresso/src/com.oracle.truffle.espresso/src/com/oracle/truffle/espresso/substitutions/Target_java_lang_invoke_VarHandles.java <==
/*
 * Copyright (c) 2020, 2021, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 */

package com.oracle.truffle.espresso.substitutions;

import java.lang.reflect.Field;

import com.oracle.truffle.api.dsl.Cached;
import com.oracle.truffle.api.dsl.Specialization;
import com.oracle.truffle.api.nodes.DirectCallNode;
import com.oracle.truffle.espresso.runtime.StaticObject;

@EspressoSubstitutions
public final class Target_java_lang_invoke_VarHandles {

    @Substitution
    abstract static class GetStaticFieldFromBaseAndOffset extends SubstitutionNode {
        abstract @JavaType(Field.class) StaticObject execute(
                        @JavaType(Object.class) StaticObject base,
                        long offset,
                        @JavaType(Class.class) StaticObject fieldType);

        @Specialization
        public static @JavaType(Field.class) StaticObject doCached(@JavaType(Object.class) StaticObject base,
                        long offset,
                        @JavaType(Class.class) StaticObject fieldType,
                        @Cached("create(getContext().getMeta().java_lang_invoke_VarHandles_getStaticFieldFromBaseAndOffset.getCallTargetNoSubstitution())") DirectCallNode original) {
            // getStaticFieldFromBaseAndOffset expects the static base to be the j.l.Class
            return (StaticObject) original.call(base.getKlass().mirror(), offset, fieldType);
        }
    }
}

==> mandrel-23.1.5.0-Final.tar.gz: mandrel-mandrel-23.1.5.0-Final/espresso/src/com.oracle.truffle.espresso/src/com/oracle/truffle/espresso/substitutions/Target_java_lang_invoke_VarHandles.java <==
/*
 * Copyright (c) 2020, 2021, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 */

package com.oracle.truffle.espresso.substitutions;

import java.lang.reflect.Field;

import com.oracle.truffle.api.dsl.Cached;
import com.oracle.truffle.api.dsl.Specialization;
import com.oracle.truffle.api.nodes.DirectCallNode;
import com.oracle.truffle.espresso.runtime.staticobject.StaticObject;

@EspressoSubstitutions
public final class Target_java_lang_invoke_VarHandles {

    @Substitution
    abstract static class GetStaticFieldFromBaseAndOffset extends SubstitutionNode {
        abstract @JavaType(Field.class) StaticObject execute(
                        @JavaType(Object.class) StaticObject base,
                        long offset,
                        @JavaType(Class.class) StaticObject fieldType);

        @Specialization
        public static @JavaType(Field.class) StaticObject doCached(@JavaType(Object.class) StaticObject base,
                        long offset,
                        @JavaType(Class.class) StaticObject fieldType,
                        @Cached("create(getContext().getMeta().java_lang_invoke_VarHandles_getStaticFieldFromBaseAndOffset.getCallTargetNoSubstitution())") DirectCallNode original) {
            // getStaticFieldFromBaseAndOffset expects the static base to be the j.l.Class
            return (StaticObject) original.call(base.getKlass().mirror(), offset, fieldType);
        }
    }
}
